
/*
	KeyboardApplet class
	
	accepts user input text
	displays statistics on typing text with qwerty and dvorak keyboards
	loads keyboard image used for qwerty and dvorak keyboards

	author: JAM (Jon A. Maxwell)
	email: jmaxwell@acm.vt.edu
	date: Mar 20 1998
	
	modifications:
				   Addition of 6 more layouts.  
	               Michael Capewell
	               smozoma@yahoo.com
	               June 19, 2005
	               
	               Added more keys and sizes, (e.g. can use Ctrl, Alt, Shift keys, putting in text  ⒈⒉⒊ etc.)
	               Path is counted from key's center
	               Logging and warnings for not found characters. 
	               valdis.vitolins@odo.lv
	               August 29, 2010
    
*/

import java.awt.*;
import javax.swing.*;
import java.applet.Applet;
import java.io.File;
import java.io.IOException;

public class KeyboardApplet extends Applet {
	
	public static final String logFile = "./typing.log";
	
	TextArea text = new TextArea(10, 80);
	Button calc = new Button("Calculate");

	Keyboard ergonomic, modern;
	KeyStats ergonomicStats, modernStats;
/*	

	String qwertyKeymap[] = {
		// lowercase, uppercase
		"`1234567890-= ",   "~!@#$%^&*()_+ "  ,
		"\tqwertyuiop[]"  ,   " QWERTYUIOP{}"   ,
		" asdfghjkl;'\\\n",   " ASDFGHJKL:\"|\n",  	// don't put spaces if won't them counted, 
		"  zxcvbnm,./ "  ,   "  ZXCVBNM<>? "   ,	// or limit range with keyboard.isSettable array
		"        "       ,   "        "
		};
	
	String adaptedKeymap[] = {
		"`1234567890[]"  ,  "~!@#$%^&*(){}",
		"\t',.pyfgcrl/="    ,  " \"<>PYFGCRL?+",
		" aoeuidhtns-\\\n"   ,  " AOEUIDHTNS_|\n",
		"  ;qjkxbmwvz "     ,  "  :QJKXBMWVZ",
		"        "               ,  "" 
		};
*/
	String ergonomicKeymap[] = {
		"`1234567890-f⒈",    "~!@#$%^&*()_F⒈",
		"\tūgjrmvnzēčžh" ,     "\tŪGJRMVNZĒČŽH" ,
		"⒉šusildatec'ķ\n" ,   "⒉ŠUSILDATEC\"Ķ\n",
		"⒊ģņbīkpoā,.ļ⒋"   ,    "⒊ĢŅBĪKPOĀ;:Ļ⒋"   ,
		"⒌⒍⒎ ⒐⒑⒒⒓ "        ,   "⒌⒍⒎ ⒐⒑⒒⒓"     
		};	

/*	
	String modernKeymap[] = {
		"`1234567890[]⒈",       "~!@#$%^&*(){}⒈",		// 14 keys
		"\tgbjkmvāzēc/=" ,     "\tGBJKMVĀZĒC?+" ,	// 13 keys Enter in not counted
		"⒉nusidlater'-\n" ,   "⒉NUSIDLATER\"_\n",	// nav slikti. 
//		"⒉rusidlaten'-\n" ,   "⒉RUSIDLATEN\"_\n",	// nav slikti. 		
		"⒊xfwhīpoy,.q⒋"   ,  "⒊XFWHĪPOY;:Q⒋"   ,	// 13 keys		
		"⒌⒍⒎ ⒐⒑⒒⒓"   ,   "⒌⒍⒎ ⒐⒑⒒⒓"			// 8 keys including 2 control keys, Space, 2 Win keys and menu key
	};	
	
*/	
/*
		String modernKeymap[] = {
			"`1234567890[]⒈",       "~!@#$%^&*(){}⒈",		// 14 keys
			"\tbēokzlājmd=" ,     "\tBĒOKZLĀJMD?+" ,		// 13 keys Enter in not counted
			"⒉usirpvnate'-\n" ,   "⒉USIRPVNATE\"_\n",		// nav slikti. 
			"⒊xwfīchgy,.q⒋"   ,  "⒊XWFĪCHGY;:Q⒋"   ,		// 13 keys		
			"⒌⒍⒎ ⒐⒑⒒⒓"   ,   "⒌⒍⒎ ⒐⒑⒒⒓"			// 8 keys including 2 control keys, Space, 2 Win keys and menu key
		};	
		
		String modernKeymap[] = {
				"`1234567890[]⒈",       "~!@#$%^&*(){}⒈",		// 14 keys
				"\tbjomlzkādē=" ,     "\tBJOMLZKĀDĒ?+" ,		// 13 keys Enter in not counted
				"⒉usirvpnate'-\n" ,   "⒉USIRVPNATE\"_\n",		// nav slikti. 
				"⒊xwfīchg,.yq⒋"   ,  "⒊XWFĪCHG;:YQ⒋"   ,		// 13 keys		
				"⒌⒍⒎ ⒐⒑⒒⒓"   ,   "⒌⒍⒎ ⒐⒑⒒⒓"			// 8 keys including 2 control keys, Space, 2 Win keys and menu key
			};				
			
*/	
		String modernKeymap[] = {
				"`1234567890[]⒈",       "~!@#$%^&*(){}⒈",		// 14 keys
				"\tbjomlzkādē=" ,     "\tBJOMLZKĀDĒ?+" ,		// 13 keys Enter in not counted
				"⒉usirvpnateyq\n" ,   "⒉USIRVPNATEYQ\n",		// nav slikti. 
				"⒊xwfīchg,.'-⒋"   ,  "⒊XWFĪCHG;:\"_⒋"   ,		// 13 keys		
				"⒌⒍⒎ ⒐⒑⒒⒓"   ,   "⒌⒍⒎ ⒐⒑⒒⒓"			// 8 keys including 2 control keys, Space, 2 Win keys and menu key
			};		

		
	//	setup applet gui, keyboards

	public void init() {
		// load images
      Image ergonomicImage = null;			
      Image modernImage = null;
      try {
         ergonomicImage = Toolkit.getDefaultToolkit().getImage( getClass().getClassLoader().getResource("keyboard.jpg"));
         modernImage = ergonomicImage;
      }
      catch (Exception ex) {
      }

		ergonomic = new Keyboard(modernImage, "ergonomic");
		modern = new Keyboard(ergonomicImage, "modern");
		for (int i=0; i < ergonomicKeymap.length; i+=2) {
			ergonomic.setRow(i/2,Keyboard.LOWERCASE, ergonomicKeymap[i]);
			ergonomic.setRow(i/2,Keyboard.UPPERCASE, ergonomicKeymap[i+1]);
		}		
		for (int i=0; i < modernKeymap.length; i+=2) {
			modern.setRow(i/2,Keyboard.LOWERCASE, modernKeymap[i]);
			modern.setRow(i/2,Keyboard.UPPERCASE, modernKeymap[i+1]);
		}		
		// stats
		try {
			ergonomicStats = new KeyStats();
			ergonomicStats.calculate("", ergonomic);
			modernStats = new KeyStats();
			modernStats.calculate("", modern);			
		} catch (Exception e) {
			e.printStackTrace();
		}


		// gui

		this.setBackground(new Color(220,220,220));
		resize(620,530);

		// I know, I know, GridbagLayout... or something
		
		GridBagLayout gridbag = new GridBagLayout();
        // GridBagConstraints c = new GridBagConstraints();
        setLayout(gridbag);

		// textPanel: text area and calculate button
		Panel calcPanel = new Panel();
		calcPanel.setLayout( new FlowLayout() );
		calcPanel.add(calc);
		//
		Panel textPanel = new Panel();
		textPanel.setLayout( new BorderLayout() );
		textPanel.add("Center",text);
		textPanel.add("South",calcPanel);

		//========================================================
		// cmodernStatsPanel: cmodern label and cmodernStats
		Panel cmodernStatsPanel = new Panel();
		Label cmodernLabel = new Label("Ergonomic", Label.LEFT);
		cmodernLabel.setFont(new Font("Dialog", Font.BOLD, 16));
		cmodernStatsPanel.setLayout( new BorderLayout() );
		cmodernStatsPanel.add("Center", ergonomicStats);
		cmodernStatsPanel.add("North", cmodernLabel);

		// cergonomicStatsPanel: cergonomic label and cergonomicStats
		Panel cergonomicStatsPanel = new Panel();
		Label cergonomicLabel = new Label("Modern", Label.LEFT);
		cergonomicLabel.setFont(new Font("Dialog", Font.BOLD, 16));
		cergonomicStatsPanel.setLayout( new BorderLayout() );
		cergonomicStatsPanel.add("Center", modernStats);
		cergonomicStatsPanel.add("North", cergonomicLabel);

		
		//========================================================
		// cmodernPanel: cmodernStatsPanel and cmodern keyboard
		Panel cmodernPanel = new Panel();
		cmodernPanel.setLayout( new BorderLayout() );
		cmodernPanel.add("North", cmodernStatsPanel);
		cmodernPanel.add("Center",ergonomic);
		
		// cergonomicPanel: cergonomicStatsPanel and cergonomic keyboard
		Panel cergonomicPanel = new Panel();
		cergonomicPanel.setLayout( new BorderLayout() );
		cergonomicPanel.add("North", cergonomicStatsPanel);
		cergonomicPanel.add("Center",modern);

		// klausler2Panel: klausler2StatsPanel and klausler2 keyboard
		Panel klausler2Panel = new Panel();
		klausler2Panel.setLayout( new BorderLayout() );
		
		// arensitoPanel: arensitoStatsPanel and arensito keyboard
		Panel arensitoPanel = new Panel();
		arensitoPanel.setLayout( new BorderLayout() );

		// arensitoPanel: c_adjacentStatsPanel and c_adjacent keyboard
		Panel c_adjacentPanel = new Panel();
		c_adjacentPanel.setLayout( new BorderLayout() );

		// c_closePanel: c_closeStatsPanel and c_close keyboard
		Panel c_closePanel = new Panel();
		c_closePanel.setLayout( new BorderLayout() );

		//========================================================
		// displayPanel: modernPanel and ergonomicPanel
		Panel displayPanel = new Panel();
		displayPanel.setLayout( new FlowLayout() );
		displayPanel.add("West",cmodernPanel);
		displayPanel.add("East",cergonomicPanel);

		// this panel: textPanel and displayPanel
		setLayout( new BorderLayout() );
		add("North",textPanel);
		add("Center",displayPanel);
		
	}

// calculate

	public boolean handleEvent(Event event) {
		if (event.id == Event.ACTION_EVENT && event.target == calc) {
			calc.disable();		
			try {
				File f = new File(logFile);
				f.delete();
				ergonomicStats.calculate(text.getText(), ergonomic);
				modernStats.calculate(text.getText(), modern);
			} catch (Exception e) {
				e.printStackTrace();
			}
			calc.enable();
			return true;
		}
		return super.handleEvent(event);
	}

}





